@@ -4,21 +4,11 @@ It emulates the signal/slot pattern from Qt, but it does not require Qt.
 """
 
 import os
+from importlib.metadata import PackageNotFoundError, version
 from typing import TYPE_CHECKING, Any
 
 if TYPE_CHECKING:
-    PackageNotFoundError = Exception
-    from ._evented_model_v1 import EventedModel  # noqa: TCH004
-
-    def version(package: str) -> str:
-        """Return version."""
-
-else:
-    # hiding this import from type checkers so mypyc can work on both 3.7 and later
-    try:
-        from importlib.metadata import PackageNotFoundError, version
-    except ImportError:
-        from importlib_metadata import PackageNotFoundError, version
+    from ._evented_model import EventedModel  # noqa: TCH004
 
 
 try:
@@ -33,8 +23,8 @@ __all__ = [
     "_compiled",
     "debounced",
     "EmissionInfo",
-    "EmitLoopError",
     "emit_queued",
+    "EmitLoopError",
     "evented",
     "EventedModel",
     "get_evented_namespace",
@@ -71,19 +61,12 @@ from ._signal import Signal, SignalInstance, _compiled
 from ._throttler import debounced, throttled
 
 
-def __getattr__(name: str) -> Any:
+def __getattr__(name: str) -> Any:  # pragma: no cover
     if name == "EventedModel":
-        import pydantic.version
-
-        if pydantic.version.VERSION.startswith("2"):
-            from ._evented_model_v2 import EventedModel
-        else:
-            from ._evented_model_v1 import EventedModel  # type: ignore
+        from ._evented_model import EventedModel
 
         return EventedModel
-    raise AttributeError(  # pragma: no cover
-        f"module {__name__!r} has no attribute {name!r}"
-    )
+    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
 
 
 del os, TYPE_CHECKING
